//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 2D            ///////
/////                                                  ///////
/////              différents schémas de VF            ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2014                    ///////
//////////////////////////////////////////////////////////////

// Toutes ces fonctions prennent deux paramètres :
//      m        : un maillage
//      donnees  : un jeu de donnees
//

//**************************************************************************
//    Routine de calcul du coefficient de diffusion aux aretes
//        selon la méthode choisie
//**************************************************************************
function [ck]=calcul_coeff_k_interf(m,donnees);
  
  if ~isfield(donnees,"methode") then
    donnees.methode="exacte";
  end;

  select donnees.methode
   case "exacte"  // On fait le calcul exact du coefficient aux centres
                  // des aretes
    
   printf('La fonction ''calcul_coeff_k_interf exacte'' n''est pas encore programmee !\n');
   abort
   
   
   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
   
     
   case "arithmetique"   // On fait la moyenne arithmétique des valeurs du
                         // coefficient aux centres des mailles voisines
  
   printf('La fonction ''calcul_coeff_k_interf arithmetique'' n''est pas encore programmee !\n');
   abort
   
   
   
   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
    
     
   case "harmonique"   // On fait la moyenne harmonique pondérée des
                       // valeurs du coefficient aux centres des mailles voisines
 
       printf('La fonction ''calcul_coeff_k_interf harmonique'' n''est pas encore programmee !\n');
   abort
   
   
   
   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
    
  else
     disp("Vous n''avez pas choisi une methode de calcul du coeff de"+...
	  " diffusion valable!");
     halt;
  end;
endfunction




//**************************************************************************
//    Le schéma VF4 
//    Prise en compte des CL de Dirichlet non-homogène (_LABEL=-1)
//                    des CL de Neumann homogène  (_LABEL<-1)
//**************************************************************************

function [A,b]=const_schema_VF4(m,donnees)

  printf("Assemblage VF4\n");
  
   printf('La fonction ''const_schema_VF4'' n''est pas encore programmee !\n');
   abort
   
   
   
   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
   
  
endfunction


//**************************************************************************
//    Le schéma DDFV pour une diffusion scalaire
//    programmé à l'aide d'opérations vectorielles
//    Prise en compte des CL de Dirichlet non-homogène (_LABEL=-1)
//                    des CL de Neumann homogène  (_LABEL<-1)
//**************************************************************************


function [A,b]=const_schema_DDFV_iso(m,donnees)

  printf("Assemblage DDFV isotrope\n");
  
   printf('La fonction ''const_schema_DDFV_iso'' n''est pas encore programmee !\n');
   abort
   
   
   
   /////////////////////////////////////////////////////////////////////
//      ##     ##  #######  #### ########     ######## ########   #######  
//      ##     ## ##     ##  ##  ##     ##       ##    ##     ## ##     ## 
//      ##     ## ##     ##  ##  ##     ##       ##    ##     ##        ## 
//      ##     ## ##     ##  ##  ########        ##    ########   #######  
//       ##   ##  ##     ##  ##  ##   ##         ##    ##        ##        
//        ## ##   ##     ##  ##  ##    ##        ##    ##        ##        
//         ###     #######  #### ##     ##       ##    ##        ######### 

   
      //////////////////////////////////////////////////////////////////////
     
endfunction


//**************************************************************************
//    Le schéma DDFV pour une diffusion matricielle (non isotrope)
//    programmé à l'aide d'opérations vectorielles
//    Prise en compte des CL de Dirichlet non-homogène (_LABEL=-1)
//                    des CL de Neumann homogène  (_LABEL<-1)
//**************************************************************************

function [A,b]=const_schema_DDFV_aniso(m,donnees)

  printf("Assemblage DDFV Anisotrope\n");

    printf('La fonction ''const_schema_DDFV_aniso'' n''est pas encore programmee !\n');
   abort
   
   
   
   /////////////////////////////////////////////////////////////////////
//      ##     ##  #######  #### ########     ######## ########   #######  
//      ##     ## ##     ##  ##  ##     ##       ##    ##     ## ##     ## 
//      ##     ## ##     ##  ##  ##     ##       ##    ##     ##        ## 
//      ##     ## ##     ##  ##  ########        ##    ########   #######  
//       ##   ##  ##     ##  ##  ##   ##         ##    ##        ##        
//        ## ##   ##     ##  ##  ##    ##        ##    ##        ##        
//         ###     #######  #### ##     ##       ##    ##        ######### 

   
      //////////////////////////////////////////////////////////////////////
      
endfunction

//**************************************************************************
//    Routine de calcul du tenseur de diffusion sur un diamant
//    selon la méthode choisie
//**************************************************************************
function [ADNN,ADNNS,ADNSNS]=calcul_coeff_mat(m,donnees);

   printf('La fonction ''calcul_coeff_mat'' n''est pas encore programmee !\n');
   abort
   
   
   
   /////////////////////////////////////////////////////////////////////
//      ##     ##  #######  #### ########     ######## ########   #######  
//      ##     ## ##     ##  ##  ##     ##       ##    ##     ## ##     ## 
//      ##     ## ##     ##  ##  ##     ##       ##    ##     ##        ## 
//      ##     ## ##     ##  ##  ########        ##    ########   #######  
//       ##   ##  ##     ##  ##  ##   ##         ##    ##        ##        
//        ## ##   ##     ##  ##  ##    ##        ##    ##        ##        
//         ###     #######  #### ##     ##       ##    ##        ######### 

   
      //////////////////////////////////////////////////////////////////////
      
endfunction

//**************************************************************************
//    Calcul de la fonction de courant associée à la solution
//    du problème elliptique calculé par VF4 sur un maillage
//    orthogonal admissible
//    Ne fonctionne pas avec DDFV !!!
//**************************************************************************
