//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 2D            ///////
/////                                                  ///////
/////                 Programme principal              ///////
/////           permettant le calcul par VF4/DDFV      ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2014                    ///////
//////////////////////////////////////////////////////////////

//**************************************************************************
// On efface toutes les variables et on charge les fonctions utiles
//**************************************************************************
clear
clc
stacksize('max');
exec('maillages2D.sci');
exec('schemas2D.sci');
exec('donnees2D.sci');


//**************************************************************************
// Saisie par l'utilisateur des données du calcul
//**************************************************************************

// choix du cas test

printf('========================================\n');
printf('Choix du cas test :\n');
for i=1:length(cas_test)
  printf('%d) %s\n',i,cas_test(i).nom);
end;

choix=-1;
while (int(choix)<=0) | (int(choix)>length(cas_test))
  choix=input("Faites votre choix :");
end;
choix=int(choix);

// Chargement des donnees

donnees=cas_test(choix);


// On regarde si les maillages sont imposés dans le jeu de données
if isfield(donnees,"maillages") then
  l=list();
  for a=donnees.maillages,
    for b=liste_maillages,
      if a==b.nom then 
	l($+1)=b;
      end;
    end;
  end;
  liste_maillages=l;
end;

// On fait un choix parmi les maillages possibles

printf('========================================\n');
printf('Choix du maillage :\n');
for i=1:length(liste_maillages)
  printf('%d) %s\n',i,liste_maillages(i).nom);
end;

choix=-1;
while (int(choix)<=0) | (int(choix)>length(liste_maillages))
  choix=input("Faites votre choix :");
end;
choix=int(choix);

maillage=liste_maillages(choix);

// choix des différentes tailles de maillage pour le trace des courbes
if isfield(maillage,"indice_min") then
  printf('========================================\n');
  printf('Choix du niveau de raffinement entre %d et %d :\n',maillage.indice_min,maillage.indice_max);
  choix=-1;
  while (int(choix)<maillage.indice_min) | (int(choix)>maillage.indice_max)
    choix=input("Faites votre choix :");
  end;
  nom_maillage=maillage.nom+'_'+string(choix);
else
  nom_maillage=maillage.nom;
end;

// si le coeff de diffusion est variable : choix de la methode

if isfield(donnees,"methode") then
  printf('========================================\n');
  printf('Choix de la méthode de calcul du coeff de diffusion :\n');
  printf('1) exacte\n');
  printf('2) moyenne arithmetique\n');
  printf('3) moyenne harmonique\n');

  choix=-1;
  while (int(choix)<=0) | (int(choix)>3)
    choix=input("Faites votre choix :");
  end;
  
  select choix
   case 1
    donnees.methode="exacte";
    donnees.methode="arithmetique";
   case 3
    donnees.methode="harmonique";
  end;
end;


// choix du schéma : VF4 ou DDFV
if isfield(donnees,"coeff_mat") then
  choix_schema=3;
  printf('========================================\n');
  printf('Schema DDFV\n');
else  
  printf('========================================\n');
  printf('Choix du schéma à utiliser :\n');
  printf('1) VF4\n');
  printf('2) DDFV isotrope\n');
  printf('3) DDFV hétérogène anisotrope\n');
  choix_schema=-1;
  while ((choix_schema<1) | (choix_schema>3))
    choix_schema=input("Faites votre choix :");
  end;
end;

printf('========================================\n');
m = lecture_maillage(rep_maillages+nom_maillage);


if isfield(donnees,"uexacte") then
   solexacte=evaluation(m.centres,donnees.uexacte);
   solexacte([m.nb_vol+1:m.nb_vol+m.nb_som]) = evaluation(m.sommets,donnees.uexacte);
end;

select choix_schema
  case 1
    [A,b]=const_schema_VF4(m,donnees);
  case 2
    [A,b]=const_schema_DDFV_iso(m,donnees);
  case 3
    [A,b]=const_schema_DDFV_aniso(m,donnees);
end;


printf('Resolution\n');
try
  // On essaie d'abord UMFPACK
  sol=umfpack(A,'\',b);
catch
  // Sinon on résout par le solveur Scilab
  sol=A\b;
end,

// TRACE DE L'ERREUR
//trace_fonction(m,sol(1:m.nb_vol)-solexacte,0,0);
//xtitle(['Erreur '; donnees.nom; m.nom]);

// TRACE DE LA SOLUTION APPROCHEE SUR LE MAILLAGE PRIMAL
trace_fonction(m,sol(1:m.nb_vol),0,0);
xtitle(['Solution approchee'; donnees.nom; m.nom]);

// TRACE DE LA SOLUTION APPROCHEE SUR LE MAILLAGE DUAL
if choix_schema>1 then
  trace_fonction_sommets(m,sol(m.nb_vol+[1:m.nb_som]),2,0);
  xtitle(['Solution approchee DUALE'; donnees.nom; m.nom]);
end;


if isfield(donnees,"uexacte") then
  printf('========================================\n');

  
  // TRACE DE LA SOLUTION EXACTE SUR LE MAILLAGE PRIMAL
  trace_fonction(m,solexacte,1,1);
  xtitle(['Solution exacte'; donnees.nom; m.nom]);
  
  // TRACE DE LA SOLUTION APPROCHEE SUR LE MAILLAGE DUAL
  if choix_schema>1 then
    trace_fonction_sommets(m,solexacte(m.nb_vol+[1:m.nb_som]),3,1);
    xtitle(['Solution exacte DUALE'; donnees.nom; m.nom]);
  end;  
  
  printf('Erreur en norme L2 = %e\n',norme_L2(m,sol(1:m.nb_vol)-solexacte(1:m.nb_vol)));
  printf('Erreur en norme Linf = %e\n',max(abs(sol(1:m.nb_vol)-solexacte(1:m.nb_vol))));
  select choix_schema
   case 1
    printf('Erreur en semi-norme H1 = %e\n',norme_H1(m,sol(1:m.nb_vol)-solexacte(1:m.nb_vol))); 
   case 2
    printf('Erreur en semi-norme H1 = %e\n',norme_H1z_DDFV(m,sol-solexacte)); 
   case 3
    printf('Erreur en semi-norme H1 = %e\n',norme_H1z_DDFV(m,sol-solexacte)); 
  end;  
end;


printf('==============  FIN  ===================\n');

//**************************************************************************
//           FIN
//**************************************************************************
